@@ -1,4 +1,4 @@
-#undef WITH_ACL
+#undef HAVE_LIBACL
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
 #undef HAVE_GETTEXT
