@@ -15,8 +15,6 @@
  */
  
 
-#include <gtk/gtk.h>
-
 #include <libintl.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -24,6 +22,7 @@
 
 #include "gtkacl.h"
 #include "gtkusergroup.h"
+#include "gtkacl-callbacks.h"
 
 
 #define MESSAGE_BOX(format, error)	\
@@ -46,52 +45,15 @@ enum {
 static gint gtk_acl_signals[LAST_SIGNAL] = { 0 };
 
 
-enum {
-	COL_TYPE = 0,
-	COL_NAME,
-	COL_ID,
-	/*COL_SUID,
-	COL_SGID,
-	COL_STICKY,*/
-	COL_READ,
-	COL_WRITE,
-	COL_EXECUTE	
-};
-
-
-enum {
-	UNIX_USER = 0,
-	UNIX_GROUP,
-	UNIX_OTHER
-};
-
-
-
 
 static void gtk_acl_class_init (GtkAclClass *class);
 static void gtk_acl_init (GtkAcl *gtk_acl);
 
+static GtkWidget *create_menu_type(GtkAcl *gtk_acl);
+static GtkWidget *create_perms_view(GtkAcl *gtk_acl, GtkTreeStore *acl_tree);
+static GtkWidget *create_frame_specials(GtkAcl *gtk_acl);
+static GtkWidget *create_frame_default(GtkAcl *gtk_acl);
 
-static void gtk_acl_load(GtkAcl *gtk_acl);
-
-
-void gtk_acl_on_add_acl(GtkWidget *button, gpointer *data);
-void gtk_acl_on_del_acl(GtkWidget *button, gpointer *data);
-static void gtk_acl_on_set_owner(GtkWidget *button, gpointer *data);
-
-
-static void gtk_acl_on_toggle_read(GtkCellRendererToggle *cell, gchar *path_str,
-	gpointer data);
-static void gtk_acl_on_toggle_write(GtkCellRendererToggle *cell, gchar *path_str,
-	gpointer data);
-static void gtk_acl_on_toggle_execute(GtkCellRendererToggle *cell, gchar *path_str,
-	gpointer data);
-static void gtk_acl_on_toggle_suid(GtkToggleButton *togglebutton, gpointer user_data);
-static void gtk_acl_on_toggle_sgid(GtkToggleButton *togglebutton, gpointer user_data);
-static void gtk_acl_on_toggle_sticky(GtkToggleButton *togglebutton, gpointer user_data);
-
-static void gtk_acl_on_avanced_toggled(GtkToggleButton *togglebutton, gpointer user_data);
-	
 
 /***** WIDGET CREATION *****/
 
@@ -129,7 +91,7 @@ static void gtk_acl_class_init (GtkAclClass *class)
 
 	object_class = (GtkObjectClass *) class;
   
-	gtk_acl_signals[ADD_ACL] = gtk_signal_new ("add_acl",
+	/*gtk_acl_signals[ADD_ACL] = gtk_signal_new ("add_acl",
 					GTK_RUN_FIRST,
 					G_OBJECT_CLASS_TYPE(object_class),
 					G_STRUCT_OFFSET (GtkAclClass, add_acl),
@@ -141,7 +103,7 @@ static void gtk_acl_class_init (GtkAclClass *class)
 					G_OBJECT_CLASS_TYPE(object_class),
 					G_STRUCT_OFFSET (GtkAclClass, del_acl),
 					gtk_signal_default_marshaller, GTK_TYPE_NONE, 
-					0);
+					0);*/
 
 	class->add_acl = NULL;
 	class->del_acl = NULL;
@@ -151,180 +113,118 @@ static void gtk_acl_class_init (GtkAclClass *class)
 //
 static void gtk_acl_init (GtkAcl *gtk_acl)
 {
-	GtkWidget *box_header, *box_main, *vbox_list, *box_special, *vbox_buttons;
-	GtkWidget *acl_view, *acl_view_groups;
+	GtkWidget *box_main, *vbox_list, *box_special, *box_buttons,
+		*box_advanced;
+	
 	GtkWidget *button;
-	GtkWidget *scrolled;
-	GtkWidget *check, *advanced;
+	GtkWidget *button_advanced;
 	
-	GtkTreeStore      *acl_tree, *acl_tree_groups;
-	GtkTreeViewColumn *col;
-	GtkCellRenderer   *cell_renderer;
 	
-	//
-	box_header   = gtk_hbox_new(FALSE, 2);
-	box_main     = gtk_vbox_new(FALSE, 2);
-	vbox_list    = gtk_vbox_new(FALSE, 2);
+	box_main     = gtk_vbox_new(FALSE, 3);
+	gtk_container_set_border_width(GTK_CONTAINER(box_main), 5);
+	
 	box_special  = gtk_hbox_new(TRUE,  3);	
-	vbox_buttons = gtk_hbox_new(TRUE,  3);
+
+	
+	//HEADER - with check acl and acl type menu
+	GtkWidget *box_header;
+	GtkWidget *check;
+	box_header   = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(box_header),  GTK_BUTTONBOX_EDGE);
+	
+	check = gtk_check_button_new_with_label(_("Acl support"));
+	gtk_widget_set_sensitive(check, FALSE);
+	gtk_acl->check_acl = check;
+	gtk_container_add(GTK_CONTAINER(box_header), GTK_WIDGET(check));
+	
+	//Menu Choice  access or default
+	GtkWidget *menu_type;
+	menu_type = create_menu_type(gtk_acl);
+	gtk_container_add(GTK_CONTAINER(box_header), GTK_WIDGET(menu_type ));
+	
+	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(box_header), FALSE, FALSE, 0);
 	
 	
-	/* USERS */
+	//PERMS VIEW
+	GtkWidget    *perms_view;
+	GtkTreeStore *acl_tree;
+
 	acl_tree = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
 		G_TYPE_BOOLEAN,	G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
 	gtk_acl->acl_data = acl_tree;
 	
-	
-	//Some buttons
-	check = gtk_check_button_new_with_label(_("Acl"));
-	gtk_widget_set_sensitive(check, FALSE);
-	gtk_acl->check_acl = check;
-	gtk_box_pack_start(GTK_BOX(box_header), GTK_WIDGET(check), TRUE, TRUE, 0);
-	
-	advanced = gtk_toggle_button_new_with_label(_("Advanced"));
-	g_signal_connect(G_OBJECT(advanced), "toggled", G_CALLBACK(gtk_acl_on_avanced_toggled), gtk_acl);
-	//gtk_widget_set_sensitive(check, FALSE);
-	gtk_acl->advanced = advanced ;
-	gtk_box_pack_start(GTK_BOX(box_header), GTK_WIDGET(advanced ), TRUE, TRUE, 0);
-
-	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(box_header), TRUE, TRUE, 0);
-
-
-	//SCrolled view
-	scrolled = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-                                  	GTK_POLICY_AUTOMATIC,
-                                  	GTK_POLICY_AUTOMATIC);
-					
+	perms_view = create_perms_view(gtk_acl, acl_tree);
+	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(perms_view), TRUE, TRUE, 0);
 	
 	
-	//Acl Tree View
-	acl_view = gtk_tree_view_new();
-	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(acl_view), TRUE);
-	gtk_tree_view_set_model(GTK_TREE_VIEW(acl_view), GTK_TREE_MODEL(acl_tree));
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(acl_view), TRUE);
-	gtk_widget_set_size_request(acl_view, 400, 250);
-	gtk_acl->acl_view = acl_view;
-	
-
-	//Column type
-	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, (gchar *) _("Type"));
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
-	cell_renderer = gtk_cell_renderer_text_new();
-	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute(col, cell_renderer, "text", COL_TYPE);
-
-	//Column Name
-	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, (gchar *) _("Name"));
-	//gtk_tree_view_column_set_resizable (col, TRUE);
-	//gtk_tree_view_column_set_sizing(col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
-	cell_renderer = gtk_cell_renderer_text_new();
-	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute(col, cell_renderer, "text", COL_NAME);
-	
-	//Column Id
-	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, (gchar *) _("Id"));
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
-	cell_renderer = gtk_cell_renderer_text_new();
-	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute(col, cell_renderer, "text", COL_ID);
-	
-	//Column Read
-	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, "Read");
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
-	cell_renderer = gtk_cell_renderer_toggle_new();
-	g_signal_connect(G_OBJECT(cell_renderer), "toggled", 
-		G_CALLBACK(gtk_acl_on_toggle_read), gtk_acl);
-	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute(col, cell_renderer, "active", COL_READ);
-	
-	//Column Write
-	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, "Write");
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
-	cell_renderer = gtk_cell_renderer_toggle_new();
-	g_signal_connect(G_OBJECT(cell_renderer), "toggled", 
-		G_CALLBACK(gtk_acl_on_toggle_write), gtk_acl);	
-	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute(col, cell_renderer, "active", COL_WRITE);
-	
-	//Column Exec
-	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, "Exec");
-	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
-	cell_renderer = gtk_cell_renderer_toggle_new();
-	g_signal_connect(G_OBJECT(cell_renderer), "toggled", 
-		G_CALLBACK(gtk_acl_on_toggle_execute), gtk_acl);
-	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute(col, cell_renderer, "active", COL_EXECUTE);
-	
-	
-	gtk_widget_show(acl_view);
-	gtk_container_add (GTK_CONTAINER(scrolled), acl_view);
-	gtk_box_pack_start(GTK_BOX(vbox_list), GTK_WIDGET(scrolled), TRUE, TRUE, 0);
+	//ADVANCED MENU
+	box_advanced  = gtk_hbox_new(FALSE,  2);	
+	gtk_container_set_border_width(GTK_CONTAINER(box_advanced), 5);
 	
+		
 	
 	//Special perms
-	GtkWidget *frame;
-	frame = gtk_frame_new(_("Specials permissions"));
-	gtk_acl->special_frame = frame;
-	//gtk_frame_set_label(frames, _("Specials permissions")));
-	
-	check = gtk_check_button_new_with_label("suid");
-	g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(gtk_acl_on_toggle_suid), 
-		gtk_acl);
-	gtk_acl->special[0] = check;
-	gtk_box_pack_start(GTK_BOX(box_special), GTK_WIDGET(check), TRUE, TRUE, 0);
+	GtkWidget *frame;	
+	frame = create_frame_specials(gtk_acl);
+	gtk_box_pack_start(GTK_BOX(box_advanced), GTK_WIDGET(frame), FALSE, FALSE, 0);
 
-	check = gtk_check_button_new_with_label("sgid");
-	g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(gtk_acl_on_toggle_sgid), 
-		gtk_acl);
-	gtk_acl->special[1] = check;
-	gtk_box_pack_start(GTK_BOX(box_special), GTK_WIDGET(check), TRUE, TRUE, 0);
+	frame = create_frame_default(gtk_acl);
+	gtk_box_pack_end(GTK_BOX(box_advanced), GTK_WIDGET(frame), FALSE, FALSE, 0);
+	
+	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(box_advanced), FALSE, FALSE, 0);
+	gtk_widget_show(box_advanced);
+	
+	
+	// ACTIONS BUTTONS
+	
+	//Action box
+	GtkWidget *box_action;
+	box_action = gtk_hbox_new(FALSE, 0);
+	
+	//Button advanced
+	button_advanced = gtk_toggle_button_new_with_label(_("Advanced"));
 
-	check = gtk_check_button_new_with_label("sticky bit");
-	g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(gtk_acl_on_toggle_sticky), 
-		gtk_acl);
-	gtk_acl->special[2] = check;
-	gtk_box_pack_start(GTK_BOX(box_special), GTK_WIDGET(check), TRUE, TRUE, 0);
+	g_signal_connect(G_OBJECT(button_advanced), "toggled", G_CALLBACK(gtk_acl_on_avanced_toggled), gtk_acl);
+	gtk_box_pack_start(GTK_BOX(box_action),  GTK_WIDGET(button_advanced), FALSE, FALSE, 20);
+	gtk_acl->advanced = button_advanced ;
+	gtk_widget_show(button_advanced);
 	
-	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(box_special));
-	gtk_box_pack_start(GTK_BOX(vbox_list), GTK_WIDGET(frame), TRUE, TRUE, 0);
-
+	//Box buttons
+	box_buttons = gtk_hbutton_box_new();
+	gtk_box_set_spacing(GTK_BOX(box_buttons), 5);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(box_buttons), GTK_BUTTONBOX_END);
 	
-	
-	/* BUTTONS */
-	
-	//Buton "Owner"
-	button = gtk_button_new_with_label(_("Set Owner"));
+	//Button owner
+	button = gtk_button_new_with_label(_("Owner"));
 	gtk_widget_show(button);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_acl_on_set_owner), 
 		gtk_acl);
-	gtk_box_pack_start(GTK_BOX(vbox_buttons), GTK_WIDGET(button), FALSE, FALSE, 5);
+	gtk_container_add(GTK_CONTAINER(box_buttons), GTK_WIDGET(button));
+	gtk_acl->button_owner = button;
 
-	//Buton "Add ACL"
-	button = gtk_button_new_from_stock(GTK_STOCK_ADD);
-	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_acl_on_add_acl), gtk_acl);
-	gtk_widget_show(button);
-	gtk_box_pack_start(GTK_BOX(vbox_buttons), GTK_WIDGET(button), FALSE, FALSE, 5);
-		
 	//Buton "Del ACL"
 	button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_acl_on_del_acl), gtk_acl);
 	gtk_widget_show(button);
-	gtk_box_pack_start(GTK_BOX(vbox_buttons), GTK_WIDGET(button), FALSE, FALSE, 5);
+	gtk_container_add(GTK_CONTAINER(box_buttons), GTK_WIDGET(button));
+	gtk_acl->button_del = button;
 	
+	//Buton "Add ACL"
+	button = gtk_button_new_from_stock(GTK_STOCK_ADD);
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_acl_on_add_acl), gtk_acl);
+	gtk_widget_show(button);
+	gtk_container_add(GTK_CONTAINER(box_buttons), GTK_WIDGET(button));
+	gtk_acl->button_add = button;
+
+	//
+	gtk_box_pack_end(GTK_BOX(box_action), GTK_WIDGET(box_buttons), FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(box_action), FALSE, FALSE, 0);
+
 	//
-	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(vbox_list), TRUE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(box_main), GTK_WIDGET(vbox_buttons), TRUE, TRUE, 0);
 	gtk_container_add (GTK_CONTAINER(gtk_acl), box_main);
 	
+	
+	
 	//Create UserGroup Window
 	gtk_acl->win_usergroup = gtk_usergroup_new();
 	gtk_usergroup_set_validate_user(GTK_USERGROUP(gtk_acl->win_usergroup), 
@@ -335,11 +235,6 @@ static void gtk_acl_init (GtkAcl *gtk_acl)
 	//Create UserGroup for owner
 	gtk_acl->win_usergroup_owner = gtk_usergroup_new();
 	
-	/*if (getuid() == 0) 
-	{
-		gtk_usergroup_hide_users(GTK_USERGROUP(gtk_acl->win_usergroup_owner));
-	}*/
-	
 	gtk_usergroup_set_validate_user(GTK_USERGROUP(gtk_acl->win_usergroup_owner), 
 		GTK_WIDGET(gtk_acl), G_CALLBACK(gtk_acl_set_user_owner));
 	gtk_usergroup_set_validate_group(GTK_USERGROUP(gtk_acl->win_usergroup_owner), 
@@ -347,11 +242,10 @@ static void gtk_acl_init (GtkAcl *gtk_acl)
 		
 
 	gtk_widget_show_all(box_main);
-	gtk_widget_hide(gtk_acl->special_frame);
+	//gtk_widget_hide(gtk_acl->special_frame);
 	
 	//
 	gtk_acl->perms = g_perms_new();
-	//gtk_acl_set_advanced_mode(GTK_ACL(gtk_acl), FALSE);
 }
 
 
@@ -365,16 +259,48 @@ GtkWidget *gtk_acl_new()
 
 
 //
-void gtk_acl_load_file(GtkAcl *gtk_acl, const gchar *filename)
+gboolean gtk_acl_load_file(GtkAcl *gtk_acl, const gchar *filename, const gint type)
 {
-	g_perms_load_file(gtk_acl->perms, filename);
+	//GtkWidget *tooltips;
+	
+	if (g_perms_load_file(gtk_acl->perms, filename, type) == FALSE)
+	{
+		MESSAGE_BOX("Wrong path!", 0);		
+		return FALSE;
+	}
+	
+	
+	gtk_widget_set_sensitive(GTK_WIDGET(gtk_acl->menu_mode), FALSE);
+	gtk_widget_set_sensitive(GTK_WIDGET(gtk_acl->frame_default), FALSE);
+	
 	
 	if (gtk_acl->perms->mode == P_MODE_ACL)
+	{
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->check_acl), TRUE);
-	else
+		if (gtk_acl->perms->is_directory == TRUE)
+		{
+			
+			
+			gtk_widget_set_sensitive(GTK_WIDGET(gtk_acl->menu_mode), TRUE);
+			
+			/*gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), GTK_WIDGET(gtk_acl->check_special[0]),
+				"The program is executed with the owner PPprivileges", 
+				"");*/
+		}
+	}
+	else 
+	{
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->check_acl), FALSE);
+		
+	}
 	
 	gtk_acl_load(gtk_acl);
+	
+	/*gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), GTK_WIDGET(gtk_acl->check_special[0]),
+				"The program is executed with the owner PPprivileges", 
+				"");*/
+
+	return TRUE;
 }
 
 
@@ -386,14 +312,16 @@ void gtk_acl_load(GtkAcl *gtk_acl)
 	GtkTreeIter iter_user, iter_group;
 	struct _entry_perms *entry_perms;
 
+	
+	gtk_tree_store_clear(gtk_acl->acl_data);
 
 	entry_perms = g_perms_get_entry(gtk_acl->perms, FIRST_ENTRY);
 	if (entry_perms == NULL) {
-		MESSAGE_BOX_ERROR();
+		//MESSAGE_BOX_ERROR();
 		return;
 	}
 
-	gtk_tree_store_clear(gtk_acl->acl_data);
+	
 
 	gtk_tree_store_append(GTK_TREE_STORE(gtk_acl->acl_data), 
 		&iter_user_root, NULL);
@@ -482,7 +410,7 @@ void gtk_acl_load(GtkAcl *gtk_acl)
 			gtk_tree_store_set(
 				GTK_TREE_STORE(gtk_acl->acl_data), 
 				&iter_others_root, 
-				COL_TYPE   , _("Other"),
+				COL_TYPE   , _("Others"),
 				COL_READ   , entry_perms->read,
 				COL_WRITE  , entry_perms->write,
 				COL_EXECUTE, entry_perms->exec,
@@ -520,12 +448,14 @@ void gtk_acl_load(GtkAcl *gtk_acl)
 	
 	special_perms = g_perms_get_special_perms(gtk_acl->perms);
 	
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->special[0]), 
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->check_special[0]), 
 		special_perms->suid);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->special[1]), 
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->check_special[1]), 
 		special_perms->sgid);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->special[2]), 
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_acl->check_special[2]), 
 		special_perms->sticky);
+	
+	gtk_acl_gray(gtk_acl);
 }
 
 
@@ -604,317 +534,262 @@ void gtk_acl_set_advanced_mode(GtkAcl *gtk_acl, const gboolean state)
 	
 	if (state == FALSE) 
 	{
-		gtk_widget_hide(gtk_acl->special[0]);
+		/*gtk_widget_hide(gtk_acl->special[0]);
 		gtk_widget_hide(gtk_acl->special[1]);
-		gtk_widget_hide(gtk_acl->special[2]);
+		gtk_widget_hide(gtk_acl->special[2]);*/
 		
-		gtk_widget_hide(gtk_acl->special_frame);
+		gtk_widget_hide(gtk_acl->frame_specials);
+		gtk_widget_hide(gtk_acl->frame_default);
+		
+		//gtk_widget_hide(gtk_acl->menu_mode);
 	}
 	
 	
 	if (state == TRUE) 
 	{
-		gtk_widget_show(gtk_acl->special[0]);
+		/*gtk_widget_show(gtk_acl->special[0]);
 		gtk_widget_show(gtk_acl->special[1]);
 		gtk_widget_show(gtk_acl->special[2]);
-		
-		gtk_widget_show(gtk_acl->special_frame);
+		*/
+		gtk_widget_show_all(gtk_acl->frame_specials);
+		gtk_widget_show_all(gtk_acl->frame_default);
+		//gtk_widget_show(gtk_acl->menu_mode);
 	}
 	
 	
 }
 
 
-
-
-/***** CALLBACKS *****/
-
 //
-void gtk_acl_on_add_acl(GtkWidget *button, gpointer *data)
+void gtk_acl_gray(GtkAcl *gtk_acl)
 {
-	GtkAcl *gtk_acl;
-	
-	gtk_acl = GTK_ACL(data);
-	gtk_widget_show(GTK_WIDGET(gtk_acl->win_usergroup));
-}
+	uid_t uid;
+	GtkTreeIter iter;
+	gchar *char_uid;
 
-
-//
-void gtk_acl_on_del_acl(GtkWidget *button, gpointer *data)
-{
-	GtkAcl *gtk_acl;
-	GtkTreeModel *tree_store;
-	GtkTreePath *path;
-	GtkTreeViewColumn *column;
-	gint *indices;
-	gint depth;
-	
-	gtk_acl= GTK_ACL(data);
-
-	gtk_tree_view_get_cursor(GTK_TREE_VIEW(gtk_acl->acl_view), &path, &column);
-	if (path == NULL)
-		return;
-	
-	indices = gtk_tree_path_get_indices(path);
-	depth   = gtk_tree_path_get_depth(path);
-
-	//User	
-	if ((indices[0] == 0) && (depth == 2)) {
-		GtkTreeSelection *selection;
-		GtkTreeIter iter;
-		gchar *id;
-		uid_t uid;
-
-		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtk_acl->acl_view));
-		if (!gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), 
-							&tree_store, &iter)) {
-			return;
+	//IF USER IS THE OWNER ALLOW ADD AND DEL
+	if (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, "0"))
+	{
+		gtk_tree_model_get(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, COL_ID, &char_uid, -1);
+		uid = atoi(char_uid);
+		if (getuid() == uid)
+		{
+			gtk_widget_set_sensitive(gtk_acl->button_add, TRUE);
+			gtk_widget_set_sensitive(gtk_acl->button_del, TRUE);
 		}
-		
-		gtk_tree_model_get(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, COL_ID, &id, -1);
-	
-		uid = (uid_t) atoi(id);
-		gtk_acl_del_user(gtk_acl, (uid_t) uid);
-		
-	}
-	
-	//Group
-	if ((indices[0] == 1) && (depth == 2)) {
-		GtkTreeSelection *selection;
-		GtkTreeIter iter;
-		gchar *id;
-		uid_t uid;
-
-		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtk_acl->acl_view));
-		if (!gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), 
-							&tree_store, &iter)) {
-			return;
+		else
+		{
+			gtk_widget_set_sensitive(gtk_acl->button_add, FALSE);
+			gtk_widget_set_sensitive(gtk_acl->button_del, FALSE);
 		}
-		
-		gtk_tree_model_get(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, COL_ID, &id, -1);
-
-		uid = (uid_t) atoi(id);
-		gtk_acl_del_group(gtk_acl, (uid_t) uid);
-		
 	}
 	
 	
+	//IF USER IS THE OWNER ALLOW ADD, DEL AND CHANGE OWNER
+	if (getuid() == 0)	/* Maybe Use geteuid ? */
+	{
+		gtk_widget_set_sensitive(gtk_acl->button_owner, TRUE);
+		gtk_widget_set_sensitive(gtk_acl->button_add, TRUE);
+		gtk_widget_set_sensitive(gtk_acl->button_del, TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive(gtk_acl->button_owner, FALSE);
+	}
+	
 
 	
 }
 
 
+/***** CONTROL CREATION *****/
+
+//
+static GtkWidget *create_menu_type(GtkAcl *gtk_acl)
+{
+	GtkWidget *options;
+	GtkWidget *menu, *menu_item;
+	menu = gtk_menu_new();
+	menu_item = gtk_menu_item_new_with_label(_("Access"));
+	g_signal_connect(G_OBJECT(menu_item), "activate", gtk_acl_on_show_access, gtk_acl);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+	menu_item = gtk_menu_item_new_with_label(_("Default"));
+	g_signal_connect(G_OBJECT(menu_item), "activate", gtk_acl_on_show_default, gtk_acl);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+	
+	options = gtk_option_menu_new();
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(options), menu);
+	gtk_acl->menu_mode = options;
+	gtk_widget_show(GTK_WIDGET(options));
+	
+	return options;
+}
+
+
 //
-static void gtk_acl_on_set_owner(GtkWidget *button, gpointer *data) 
+static GtkWidget *create_perms_view(GtkAcl *gtk_acl, GtkTreeStore *acl_tree)
 {
-	GtkAcl *gtk_acl;
+	GtkTreeViewColumn *col;
+	GtkCellRenderer   *cell_renderer;
+	GtkWidget         *scrolled;
+	GtkWidget         *acl_view; 
+	
+	//SCrolled view
+	scrolled = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+                                  	GTK_POLICY_AUTOMATIC,
+                                  	GTK_POLICY_AUTOMATIC);
+					
+	
+	
+	//Acl Tree View
+	acl_view = gtk_tree_view_new();
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(acl_view), TRUE);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(acl_view), GTK_TREE_MODEL(acl_tree));
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(acl_view), TRUE);
+	gtk_widget_set_size_request(acl_view, 400, 250);
+	gtk_acl->acl_view = acl_view;
 	
-	gtk_acl = GTK_ACL(data);
-	gtk_widget_show(GTK_WIDGET(gtk_acl->win_usergroup_owner));
-}
 
+	//Column type
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, (gchar *) _("Type"));
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
+	cell_renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, cell_renderer, "text", COL_TYPE);
 
-//
-static void gtk_acl_on_toggle_read(GtkCellRendererToggle *cell, gchar *path_str,
-	gpointer data)
-{
-	GtkAcl *gtk_acl;
-	GtkTreeIter iter;
-	GtkTreePath *path;
+	//Column Name
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, (gchar *) _("Name"));
 	
-	gchar    *id;
-	gboolean  state;	
-	gint *indices;
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
+	cell_renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, cell_renderer, "text", COL_NAME);
+	
+	//Column Id
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, (gchar *) _("Id"));
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
+	cell_renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, cell_renderer, "text", COL_ID);
+	
+	//Column Read
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, _("Read"));
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
+	cell_renderer = gtk_cell_renderer_toggle_new();
+	g_signal_connect(G_OBJECT(cell_renderer), "toggled", 
+		G_CALLBACK(gtk_acl_on_toggle_read), gtk_acl);
+	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, cell_renderer, "active", COL_READ);
+	
+	//Column Write
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, _("Write"));
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
+	cell_renderer = gtk_cell_renderer_toggle_new();
+	g_signal_connect(G_OBJECT(cell_renderer), "toggled", 
+		G_CALLBACK(gtk_acl_on_toggle_write), gtk_acl);	
+	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, cell_renderer, "active", COL_WRITE);
+	
+	//Column Exec
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title(col, _("Exec"));
+	gtk_tree_view_append_column(GTK_TREE_VIEW(acl_view), col);
+	cell_renderer = gtk_cell_renderer_toggle_new();
+	g_signal_connect(G_OBJECT(cell_renderer), "toggled", 
+		G_CALLBACK(gtk_acl_on_toggle_execute), gtk_acl);
+	gtk_tree_view_column_pack_start(col, cell_renderer, TRUE);
+	gtk_tree_view_column_add_attribute(col, cell_renderer, "active", COL_EXECUTE);
 	
 	
-	gtk_acl = GTK_ACL(data);
+	gtk_widget_show(acl_view);
+	gtk_container_add (GTK_CONTAINER(scrolled), acl_view);
+	//gtk_box_pack_start(GTK_BOX(vbox_list), GTK_WIDGET(scrolled), TRUE, TRUE, 0);
 	
-	path = gtk_tree_path_new_from_string(path_str);
-	gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, path);
+	return scrolled;
 	
-	gtk_tree_model_get(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, 
-		COL_ID, &id, 
-		COL_READ, &state, -1);
-	
-	indices = gtk_tree_path_get_indices(path);
-	
-	
-	//For unix user
-	if (indices[0] == 0)
-		g_perms_set_perm(gtk_acl->perms, P_USER, (uid_t) atoi(id),
-			P_READ, !state);
-		
-	//For Groups
-	if (indices[0] == 1)
-		g_perms_set_perm(gtk_acl->perms, P_GROUP, (uid_t) atoi(id), 
-			P_READ, !state);
-
-	//For Others
-	if (indices[0] == 2)
-		g_perms_set_perm(gtk_acl->perms, P_OTHER, 0, P_READ, !state);
-		
-	//For Mask
-	if (indices[0] == 3)
-		g_perms_set_perm(gtk_acl->perms, P_MASK, 0, P_READ, !state);
-			
-	
-	gtk_acl_load(gtk_acl);
-}
-
-
-//
-static void gtk_acl_on_toggle_write(GtkCellRendererToggle *cell, gchar *path_str,
-	gpointer data)
-{
-	GtkAcl *gtk_acl;
-	GtkTreeIter iter;
-	GtkTreePath *path;
-	
-	gchar    *id;
-	gboolean  state;	
-	gint *indices;
-	
-	
-	gtk_acl = GTK_ACL(data);
-	
-	path = gtk_tree_path_new_from_string(path_str);
-	gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, path);
-	
-	gtk_tree_model_get(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, 
-		COL_ID, &id, 
-		COL_WRITE, &state, -1);
-	
-	indices = gtk_tree_path_get_indices(path);
-	
-	
-	//For unix user
-	if (indices[0] == 0)
-		g_perms_set_perm(gtk_acl->perms, P_USER, (uid_t) atoi(id),
-			P_WRITE, !state);
-		
-	//For Groups
-	if (indices[0] == 1)
-		g_perms_set_perm(gtk_acl->perms, P_GROUP, (uid_t) atoi(id), 
-			P_WRITE, !state);
-
-	//For Others
-	if (indices[0] == 2)
-		g_perms_set_perm(gtk_acl->perms, P_OTHER, 0, P_WRITE, !state);
-		
-	//For Mask
-	if (indices[0] == 3)
-		g_perms_set_perm(gtk_acl->perms, P_MASK, 0, P_WRITE, !state);
-			
-	
-	gtk_acl_load(gtk_acl);
-}
-
-
-//
-static void gtk_acl_on_toggle_execute(GtkCellRendererToggle *cell, gchar *path_str,
-	gpointer data)
-{
-	GtkAcl *gtk_acl;
-	GtkTreeIter iter;
-	GtkTreePath *path;
-	
-	gchar    *id;
-	gboolean  state;	
-	gint *indices;
-	
-	
-	gtk_acl = GTK_ACL(data);
-	
-	path = gtk_tree_path_new_from_string(path_str);
-	gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, path);
-	
-	gtk_tree_model_get(GTK_TREE_MODEL(gtk_acl->acl_data), &iter, 
-		COL_ID, &id, 
-		COL_EXECUTE, &state, -1);
-	
-	indices = gtk_tree_path_get_indices(path);
-	
-	
-	//For unix user
-	if (indices[0] == 0)
-		g_perms_set_perm(gtk_acl->perms, P_USER, (uid_t) atoi(id),
-			P_EXECUTE, !state);
-		
-	//For Groups
-	if (indices[0] == 1)
-		g_perms_set_perm(gtk_acl->perms, P_GROUP, (uid_t) atoi(id), 
-			P_EXECUTE, !state);
-
-	//For Others
-	if (indices[0] == 2)
-		g_perms_set_perm(gtk_acl->perms, P_OTHER, 0, P_EXECUTE, !state);
-		
-	//For Mask
-	if (indices[0] == 3)
-		g_perms_set_perm(gtk_acl->perms, P_MASK, 0, P_EXECUTE, !state);
-			
-	
-	gtk_acl_load(gtk_acl);
-}
-
-
-//
-static void gtk_acl_on_toggle_suid(GtkToggleButton *togglebutton, gpointer user_data)
-{
-	GtkAcl   *gtk_acl;
-	gboolean  state;
-	struct _special_entry_perms *special_entry_perms;
-	
-	gtk_acl = GTK_ACL(user_data);
-	
-	state = gtk_toggle_button_get_active(togglebutton);
-	g_perms_set_special_perms(gtk_acl->perms, P_SUID, state);
-	
-	special_entry_perms = g_perms_get_special_perms(gtk_acl->perms);
-	gtk_toggle_button_set_active(togglebutton, special_entry_perms->suid);
 }
 
 //
-static void gtk_acl_on_toggle_sgid(GtkToggleButton *togglebutton, gpointer user_data)
+static GtkWidget *create_frame_specials(GtkAcl *gtk_acl)
 {
-	GtkAcl   *gtk_acl;
-	gboolean  state;
-	struct _special_entry_perms *special_entry_perms;
+	GtkWidget *frame;
+	GtkWidget *box_specials;
+	GtkWidget *check;
+	GtkWidget *tooltips;
 	
-	gtk_acl = GTK_ACL(user_data);
 	
-	state = gtk_toggle_button_get_active(togglebutton);
-	g_perms_set_special_perms(gtk_acl->perms, P_SGID, state);
+	frame = gtk_frame_new(_("Specials permissions"));
+	gtk_acl->frame_specials = frame;
 	
-	special_entry_perms = g_perms_get_special_perms(gtk_acl->perms);
-	gtk_toggle_button_set_active(togglebutton, special_entry_perms->sgid);
+	box_specials = gtk_hbox_new(TRUE, 5);
+	
+	tooltips = gtk_tooltips_new();
+	
+	check = gtk_check_button_new_with_label("suid");
+	g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(gtk_acl_on_toggle_suid), 
+		gtk_acl);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), GTK_WIDGET(check),
+		_("Executable : the program is executed\nwith the owner privileges"), 
+		"");
+	gtk_acl->check_special[0] = check;
+	gtk_box_pack_start(GTK_BOX(box_specials), GTK_WIDGET(check), FALSE, FALSE, 2);
+
+	check = gtk_check_button_new_with_label("sgid");
+	g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(gtk_acl_on_toggle_sgid), 
+		gtk_acl);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), GTK_WIDGET(check),
+		_("Executable : the program is executed\nwith the group privileges\n"
+		"Directory  : all new file in directory\nwill be writable for group of directory"), 
+		"");
+	gtk_acl->check_special[1] = check;
+	gtk_box_pack_start(GTK_BOX(box_specials), GTK_WIDGET(check), FALSE, FALSE, 2);
+
+	check = gtk_check_button_new_with_label("sticky");
+	g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(gtk_acl_on_toggle_sticky), 
+		gtk_acl);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), GTK_WIDGET(check),
+		_("Executable : the program stay in memory after started.\nIt will start also faster\n"
+		"Directory  : only edit your files"), 
+		"");
+	gtk_acl->check_special[2] = check;
+	gtk_box_pack_start(GTK_BOX(box_specials), GTK_WIDGET(check), FALSE, FALSE, 2);
+	
+	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(box_specials));
+	
+	return frame;
 }
 
 //
-static void gtk_acl_on_toggle_sticky(GtkToggleButton *togglebutton, gpointer user_data)
+static GtkWidget *create_frame_default(GtkAcl *gtk_acl)
 {
-	GtkAcl   *gtk_acl;
-	gboolean  state;
-	struct _special_entry_perms *special_entry_perms;
+	GtkWidget *frame;
+	GtkWidget *box;
+	GtkWidget *button;
 	
-	gtk_acl = GTK_ACL(user_data);
+	frame = gtk_frame_new(_("Default"));
+	gtk_acl->frame_default = frame;
 	
-	state = gtk_toggle_button_get_active(togglebutton);
-	g_perms_set_special_perms(gtk_acl->perms, P_STICKY, state);
+	box = gtk_hbox_new(FALSE, 2);
 	
-	special_entry_perms = g_perms_get_special_perms(gtk_acl->perms);
-	gtk_toggle_button_set_active(togglebutton, special_entry_perms->sticky);
-	
-}
+	//Add Default
+	button = gtk_button_new_with_label(_("init"));
+	//gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+	g_signal_connect(G_OBJECT(button), "clicked", gtk_acl_on_init_default, gtk_acl);
+	gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(button), FALSE, FALSE, 1);
 
-
-//
-static void gtk_acl_on_avanced_toggled(GtkToggleButton *togglebutton, gpointer user_data)
-{
-	GtkWidget *gtk_acl;
+	//Del default
+	button = gtk_button_new_with_label(_("del"));
+	//gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+	g_signal_connect(G_OBJECT(button), "clicked", gtk_acl_on_clear, gtk_acl);
+	gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(button), FALSE, FALSE, 1);
 	
-	gtk_acl = GTK_WIDGET(user_data);
-	
-	gtk_acl_set_advanced_mode(GTK_ACL(gtk_acl), gtk_toggle_button_get_active(togglebutton))	;
+	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(box));
 	
+	return frame;
 }
